@@ -2,100 +2,98 @@ package Cwd::Ext;
 use strict;
 use Exporter;
 use Carp;
-use Cwd;
-
-use vars qw(@ISA @EXPORT_OK $VERSION @EXPORT $VERSION);
+use vars qw(@ISA @EXPORT_OK $VERSION @EXPORT $VERSION $DEBUG %EXPORT_TAGS);
 @ISA = qw/Exporter/;
-push @EXPORT_OK,  qw(abs_path_is_in abs_path_is_in_nd abs_path_nd);
-our $VERSION = sprintf "%d.%02d", q$Revision: 1.3 $ =~ /(\d+)/g;
-$Cwd::Ext::DEBUG =0;
-sub DEBUG : lvalue { $Cwd::Ext::DEBUG }
+@EXPORT_OK = qw(abs_path_is_in abs_path_is_in_nd abs_path_nd abs_path_matches_into symlinks_supported);
+%EXPORT_TAGS = ( all => \@EXPORT_OK );
+$VERSION = sprintf "%d.%02d", q$Revision: 1.5 $ =~ /(\d+)/g;
+
 
 sub abs_path_nd {   
-   my $absPath = shift;
-    return $absPath if $absPath =~ m{^/$};
+   my $abs_path = shift;
+   return $abs_path if $abs_path=~m{^/$};
    
-   $absPath=~/^\// or $absPath = cwd()."/$absPath";
+   unless( $abs_path=~/^\// ){
+      require Cwd;
+      $abs_path = Cwd::cwd()."/$abs_path";
+   }
     
-    my @elems = split m{/}, $absPath;
+    my @elems = split m{/}, $abs_path;
     my $ptr = 1;
-    while($ptr <= $#elems)
-    {
-        if($elems[$ptr] eq q{})
-        {
+    while($ptr <= $#elems){
+        if($elems[$ptr] eq ''      ){
             splice @elems, $ptr, 1;
         }
-        elsif($elems[$ptr] eq q{.})
-        {
+
+        elsif($elems[$ptr] eq '.'  ){
             splice @elems, $ptr, 1;
         }
-        elsif($elems[$ptr] eq q{..})
-        {
-            if($ptr < 2)
-            {
+
+        elsif($elems[$ptr] eq '..' ){
+            if($ptr < 2){
                 splice @elems, $ptr, 1;
             }
-            else
-            {
+            else {
                 $ptr--;
                 splice @elems, $ptr, 2;
             }
         }
-        else
-        {
+        else {
             $ptr++;
         }
     }
+
     return $#elems ? join q{/}, @elems : q{/};
 }
 
 
+sub abs_path_matches_into {
+   my($child,$parent)=@_;
+   defined $child  or die('missing child');
+   defined $parent or die('missing parent');
+   
+   if($child eq $parent){
+      warn(" - args are the same, returning true") if $DEBUG;
+      return $child;
+   }
+
+   # WE DON'T WANT /home/hi to match on /home/hithere 
+   unless( $child=~/^$parent\// ){
+      warn (" -[$child] is not a child of [$parent]") if $DEBUG;
+      return 0;
+   }
+   return $child;
+}  
+
 sub abs_path_is_in {
    my($child,$parent) = @_;
-   defined $child or confess('missing child path argument');
+   defined $child  or confess('missing child path argument');
    defined $parent or confess('missing parent path argument');
-
+   
+   require Cwd;
    my $_child  = Cwd::abs_path($child)  or warn("cant normalize child [$child]") and return;
    my $_parent = Cwd::abs_path($parent) or warn("cant normalize parent [$parent]") and return;
 
-   
-   if ($_child eq $_parent){
-      warn(" -[$_child] same as [$_parent]");
-      return 1;
-   }   
-   
-   # WE DONT WANT /home/hi to match on /home/hithere 
-   unless( $_child=~/^$_parent\// ){
-      warn (" -[$_child] is not a child of [$_parent]") if DEBUG;
-      return 0;
-   }
-   return $_child;
+   return abs_path_matches_into($_child,$_parent);
 }
 
 
 sub abs_path_is_in_nd {
    my($child,$parent) = @_;
-   defined $child or confess('missing child path argument');
+   defined $child  or confess('missing child path argument');
    defined $parent or confess('missing parent path argument');
 
    my $_child  = Cwd::Ext::abs_path_nd($child)  or warn("cant normalize child [$child]") and return;
    my $_parent = Cwd::Ext::abs_path_nd($parent) or warn("cant normalize parent [$parent]") and return;
 
-   
-   if ($_child eq $_parent){
-      warn(" -[$_child] same as [$_parent]");
-      return 1;
-   }   
-   
-   # WE DONT WANT /home/hi to match on /home/hithere 
-   unless( $_child=~/^$_parent\// ){
-      warn (" -[$_child] is not a child of [$_parent]");
-      return 0;
-   }
-   return $_child;
+   return abs_path_matches_into($_child,$_parent);
 }
 
 
+sub symlinks_supported {
+   return eval { symlink("",""); 1 }
+}
+
 1;
 
 __END__
@@ -104,7 +102,20 @@ __END__
 
 =head1 NAME
 
-Cwd::Ext - extended file path subroutines
+Cwd::Ext - no symlink dereference
+
+=head1 SYNOPSIS
+
+Let's imagine that '/home/myself/stuff/music' is a soft link to '/home/myself/documents/music', and our
+current working directory is '/home/myself'..
+
+   use Cwd::Ext ':all';
+   
+   abs_path_nd('./stuff/music'); # returns /home/myself/stuff/music
+
+   abs_path_is_in_nd( '/home/myself/stuff/music', '/home/myself/stuff' ); # returns true, /home/myself/stuff/music
+
+
 
 =head1 DESCRIPTION
 
@@ -117,21 +128,19 @@ Questions like,
 Unlike with Cwd, this module is in baby stage. So it is NOT tweaked for OS2, NT, etc etc.
 This is developed under POSIX.
 
-Nothing is imported by default. You must explicitely import..
+This module does not export by default. You must explicitely import what you want to use.
 
-=head1 SYNOPSIS
-
-   use Cwd::Ext qw(abs_path_is_in_nd abs_path_is_in abs_path_nd);
+=head1 SUBS
 
 =head2 abs_path_nd()
 
-Works just like Cwd::abs_path , only it does (n)o (s)ymlink (d)ereference.
-
+Works just like Cwd::abs_path , only it does (n)o symlink (d)ereference.
 
 =head2 abs_path_is_in()
 
 Arguments are child path in question, parent path to test against.
-Returns boolean. 
+Returns resolved abs path of child if yes, false if no.
+
 Will confess if missing arguments.
 If either path can't be resolved, warns and returns undef.
 
@@ -150,7 +159,16 @@ If both paths resolve to same place, returns 1 and warns.
 =head2 abs_path_is_in_nd()
 
 Same as abs_path_is_within() but does not resolve symlinks.
-If either path can't be resolved, warns and returns undef.
+
+=head2 abs_path_matches_into()
+
+Arg is child abs path, and parent abs path.
+Does not use Cwd to resolve anything, this
+just performs a substring match, returns boolean
+
+=head2 symlinks_supported()
+
+Does an eval to check if this machine supports symlinks.
 
 =head1 CAVEATS
 
@@ -158,7 +176,12 @@ This module is in ALPHA state. Needs feedback.
 
 =head1 TODO
 
-I want this to inherit Cwd. So that cwd(), etc are exported.
+I want this to inherit Cwd. So that cwd(), etc are exported. Currently, you have to use Cwd and Cwd::Ext to
+access both these subs and Cwd subs.
+
+=head1 SEE ALSO
+
+L<Cwd>
 
 =head1 AUTHOR
 
